from rest_framework.test import APITransactionTestCase

from program.tests import ImageMixin, SteeringTestCaseMixin, UserMixin


class ImageViewTestCase(UserMixin, ImageMixin, SteeringTestCaseMixin, APITransactionTestCase):
    reset_sequences = True

    def test_create_image(self):
        client = self._get_client(self.user_admin)
        url = self._url("images")
        image_data = {"image": self._get_image_file()}

        response = client.post(url, data=image_data, format="multipart")

        self.assertEqual(response.status_code, 201)

    def test_destroy_image(self):
        client = self._get_client(self.user_admin)
        image = self._create_image(owner=self.user_admin)
        url = self._url("images", image.id)

        response = client.delete(url)

        self.assertEqual(response.status_code, 204)

    def test_destroy_image_not_found_for_different_user(self):
        client = self._get_client(self.user_admin)
        image = self._create_image(owner=self.user_common)
        url = self._url("images", image.id)

        response = client.delete(url)

        self.assertEqual(response.status_code, 404)

    def test_list_images(self):
        client = self._get_client(self.user_admin)
        url = self._url("images")
        for _ in range(3):
            self._create_image(owner=self.user_admin)

        response = client.get(url)

        self.assertEqual(len(response.data), 3)

    def test_list_images_for_different_user(self):
        client = self._get_client(self.user_admin)
        url = self._url("images")
        for _ in range(3):
            self._create_image(owner=self.user_common)

        response = client.get(url)

        self.assertEqual(len(response.data), 0)

    def test_retrieve_image(self):
        client = self._get_client(self.user_admin)
        data = {"alt_text": "ALT_TEXT", "credits": "CREDITS"}
        image = self._create_image(**data, owner=self.user_admin)
        url = self._url("images", image.id)

        response = client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["alt_text"], data["alt_text"])
        self.assertEqual(response.data["credits"], data["credits"])

    def test_retrieve_image_not_found_for_different_user(self):
        client = self._get_client(self.user_admin)
        image = self._create_image(owner=self.user_common)
        url = self._url("images", image.id)

        response = client.get(url)

        self.assertEqual(response.status_code, 404)

    def test_update_alt_text(self):
        client = self._get_client(self.user_admin)
        image = self._create_image(owner=self.user_admin)
        url = self._url("images", image.id)
        update = {"alt_text": "ALT_TEXT"}

        response = client.put(url, data=update)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["alt_text"], update["alt_text"])

    def test_update_alt_text_not_found_for_different_user(self):
        client = self._get_client(self.user_admin)
        image = self._create_image(owner=self.user_common)
        url = self._url("images", image.id)

        response = client.put(url, data={"alt_text": "ALT_TEXT"})

        self.assertEqual(response.status_code, 404)

    def test_update_credits(self):
        client = self._get_client(self.user_admin)
        image = self._create_image(owner=self.user_admin)
        url = self._url("images", image.id)
        update = {"credits": "CREDITS"}

        response = client.put(url, data=update)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["credits"], update["credits"])

    def test_update_credits_not_found_for_different_user(self):
        client = self._get_client(self.user_admin)
        image = self._create_image(owner=self.user_common)
        url = self._url("images", image.id)

        response = client.put(url, data={"credits": "CREDITS"})

        self.assertEqual(response.status_code, 404)

    def test_update_ppoi(self):
        client = self._get_client(self.user_admin)
        image = self._create_image(owner=self.user_admin)
        url = self._url("images", image.id)
        update = {"ppoi": "0.3x0.7"}

        response = client.put(url, data=update)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data["ppoi"], update["ppoi"])

    def test_update_ppoi_not_found_for_different_user(self):
        client = self._get_client(self.user_admin)
        image = self._create_image(owner=self.user_common)
        url = self._url("images", image.id)

        response = client.put(url, data={"ppoi": "0.3x0.7"})

        self.assertEqual(response.status_code, 404)
